//Modify date: 2008/01/28
//Purpose : Take Image and show image
//Method:
//          (1) Call BCRCom.BCR_IMAGER_ENABLE(out Result) to enable image capture
//              return error code "Result"
//
//          (2) Call BCRCom.TakeSnapshot(out Result) to take snapshot image capture, by setting
//                Set "BCRCom.dwImgBottom" for image bottom  
//                Set "BCRCom.dwImgTop" for image top
//                Set "BCRCom.dwImgLeft" for image left
//                Set "BCRCom.dwImgRight" for image right
//                Set "BCRCom.hShapWnd" to pass a image component handle to module 
//                Set "BCRCom.dwImgFileFormat" as
//                JPEGFile = 1;
//                BMPFile = 3;
//                TIFFFile = 4;
//                Set "BCRCom.dwImgBitPerPiexl" as
//                BitPerPiexl_1 = 1;
//                BitPerPiexl_4 = 4;
//                BitPerPiexl_8 = 8;
//              return error code "Result"
//          (3) Call BCRCom.BCR_STOP_IMG(out Result) to stop image transforming
//              return error code "Result"
//          (4) When taking image, you can query
//              "BCRCom.GetImgState" to know the status, like
//                  "take picture process." means that it's taking image
//                  "take picture done." means that job is done
//                  "take picture cancel." means that job is canceled     
//              "BCRCom.dwImgTotalPercent" to know the total process percentage 
//              "BCRCom.dwImgPercent" to know the percentage of taking image      
//          (5) Call BCRCom.BCR_TRIGGER(Trigger) to send trigger to BCR module, in this case
//              "BCR_TRIG_PRESS" = 2 send key down trigger
//              "BCR_TRIG_RELEASE" = 1 send key up trigger         
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;
using BCRServiceCOMLib;

namespace TakeSnapShot
{
    public partial class Form1 : Form
    {
        BCRServiceCOMLib.BCRClass BCRCom = new BCRServiceCOMLib.BCRClass();
        uint BCR_TRIG_PRESS = 2;
        uint BCR_TRIG_RELEASE = 1;        
        public Form1()
        {
            InitializeComponent();
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            BPP_CMBO.SelectedIndex = 0;
            Type_CMBO.SelectedIndex = 0;
            this.KeyPreview = true;
            this.Focus();
            Thread newThread = new Thread(new ThreadStart(this.ThreadFunc));
            newThread.IsBackground = true;
            newThread.Start();
        }              
        private void button1_Click(object sender, EventArgs e)
        {
            uint Result = 0;           
            BCRCom.BCR_IMAGER_ENABLE(out Result);
            BCRCom.dwSnapShot = 1;
            BCRCom.dwImageHeight =  Convert.ToUInt32(Height_Text.Text);
            BCRCom.dwImageWide = Convert.ToUInt32(Width_Text.Text);
            BCRCom.hShapWnd = pictureBox1.Handle;            
            if (Type_CMBO.SelectedIndex == 0)
            {
                BCRCom.dwImgFileFormat = 1;
            }
            else if (Type_CMBO.SelectedIndex == 1)
            {
                BCRCom.dwImgFileFormat = 3;
            }
            else if (Type_CMBO.SelectedIndex == 2)
            {
                BCRCom.dwImgFileFormat = 4;
            }
            if (BPP_CMBO.SelectedIndex == 0)
            {
                BCRCom.dwImgBitPerPiexl = 1;
            }
            else if (BPP_CMBO.SelectedIndex == 1)
            {
                BCRCom.dwImgBitPerPiexl = 4;
            }
            else if (BPP_CMBO.SelectedIndex == 2)
            {
                BCRCom.dwImgBitPerPiexl = 8;
            }            
            BCRCom.TakeSnapshot(out Result);
        }
        private void button2_Click(object sender, EventArgs e)
        {
            uint Result = 0;
            BCRCom.BCR_STOP_IMG(out Result);
        }
        private void Exit_Click(object sender, EventArgs e)
        {
            EventWaitHandle hExit = new EventWaitHandle(false, EventResetMode.AutoReset, "Exit");
            hExit.Set();
            this.Close();
            Application.Exit();
        }
        public void StatusUpdate(object sender, EventArgs e)
        {
            this.ImgStatus.Text = BCRCom.GetImgState;
            this.ImgStatus.Invalidate();
        }
        public void BarMax(object sender, EventArgs e)
        {
            this.progressBar1.Maximum = Convert.ToInt32( BCRCom.dwImgTotalPercent);
            this.progressBar1.Invalidate();
        }
        public void BarUpdate(object sender, EventArgs e)
        {
            this.progressBar1.Value = Convert.ToInt32(BCRCom.dwImgPercent);
            this.progressBar1.Invalidate();
        }
        private void ThreadFunc()
        {
         
            EventWaitHandle hQuit = new Form1.EventWaitHandle(false,
                EventResetMode.AutoReset, "Exit");
            //Set up the named events.
            EventWaitHandle hKeyDown = new EventWaitHandle(false,
                 EventResetMode.AutoReset, "Event_BCR_Trigger_KeyDown");
            EventWaitHandle hKeyUp = new EventWaitHandle(false,
                 EventResetMode.AutoReset, "Event_BCR_Trigger_KeyUp");
            EventWaitHandle hImgMsg = new EventWaitHandle(false,
                 EventResetMode.AutoReset, "TAKE_IMAGE_MSG");
            EventWaitHandle hImgPercent = new EventWaitHandle(false,
                 EventResetMode.AutoReset, "TAKE_IMAGE_MSG_PERCENT");
            EventWaitHandle hImgTatal = new EventWaitHandle(false,
                 EventResetMode.AutoReset, "TAKE_IMAGE_MSG_TOTAL_BLOCK");
                                   
            while (true)
            {
                try
                {
                    int eventIndex = EventWaitHandle.WaitAny(new WaitHandle[] {
                        hQuit, hKeyDown,hKeyUp,hImgMsg,hImgPercent,hImgTatal });                  
                    switch (eventIndex)
                    {
                        case 0:
                            //event to kill the thread
                            Console.WriteLine("[kill thread]");                            
                            return;
                        case 1:                            
                            BCRCom.BCR_TRIGGER(BCR_TRIG_PRESS);                           
                            break;
                        case 2:                            
                            BCRCom.BCR_TRIGGER(BCR_TRIG_RELEASE);                           
                            break;
                        case 3:
                            ImgStatus.Invoke(new EventHandler(StatusUpdate)); 
                            break;
                        case 4:
                            progressBar1.Invoke(new EventHandler(BarUpdate));
                            break;
                        case 5:
                            progressBar1.Invoke(new EventHandler(BarMax));
                            break;                            
                    }
                }
                catch (ThreadAbortException e)
                {
                    Console.WriteLine("Thread - caught ThreadAbortException - resetting.");
                    Console.WriteLine("Exception message: {0}", e.Message);
                }

            }
        }

        internal static class NativeMethods
        {
            internal enum EVENT
            {
                PULSE = 1,
                RESET = 2,
                SET = 3,
            }

            [DllImport("coredll", SetLastError = true)]
            [return: MarshalAs(UnmanagedType.Bool)]
            internal static extern bool EventModify(IntPtr hEvent, EVENT ef);

            [DllImport("coredll", SetLastError = true)]
            internal static extern IntPtr CreateEvent(IntPtr lpEventAttributes,
            bool bManualReset, bool bInitialState, string lpName);

            [DllImport("coredll", SetLastError = true)]
            [return: MarshalAs(UnmanagedType.Bool)]
            internal static extern bool CloseHandle(IntPtr hObject);

            [DllImport("coredll", SetLastError = true)]
            internal static extern int WaitForSingleObject(IntPtr hHandle,
                int dwMilliseconds);

            [DllImport("coredll", SetLastError = true)]
            internal static extern int WaitForMultipleObjects(int nCount,
                IntPtr[] lpHandles, bool fWaitAll, int dwMilliseconds);

        }

        public class EventWaitHandle : WaitHandle
        {
            public const int WaitTimeout = 0x102;

            public EventWaitHandle(bool initialState, EventResetMode mode,
                string name)
            {
                this.Handle = NativeMethods.CreateEvent(IntPtr.Zero,
                     mode == EventResetMode.ManualReset, initialState, name);
            }

            public bool Set()
            {
                return NativeMethods.EventModify(this.Handle,
                    NativeMethods.EVENT.SET);
            }

            public bool Reset()
            {
                return NativeMethods.EventModify(this.Handle,
                    NativeMethods.EVENT.RESET);
            }

            public static int WaitAny(WaitHandle[] waitHandles)
            {
                return WaitAny(waitHandles, Timeout.Infinite, false);
            }

            public static int WaitAny(WaitHandle[] waitHandles,
                int millisecondsTimeout, bool exitContext)
            {
                IntPtr[] handles = new IntPtr[waitHandles.Length];
                for (int i = 0; i < handles.Length; i++)
                {
                    handles[i] = waitHandles[i].Handle;
                }

                return NativeMethods.WaitForMultipleObjects(handles.Length,
                    handles, false, millisecondsTimeout);
            }


            public override bool WaitOne()
            {
                return WaitOne(Timeout.Infinite, false);
            }

            public override bool WaitOne(int millisecondsTimeout, bool exitContext)
            {
                return NativeMethods.WaitForSingleObject(this.Handle,
                    millisecondsTimeout) == 0;
            }

            public override void Close()
            {
                if (this.Handle != WaitHandle.InvalidHandle)
                {
                    NativeMethods.CloseHandle(this.Handle);
                    this.Handle = WaitHandle.InvalidHandle;
                }
            }
        }

        public enum EventResetMode
        {
            AutoReset = 0,
            ManualReset = 1,
        }                  
    }
}